PROGRAM_NAME='PSTN'
/////////////////////////////////////////////////////////////
//                                                         //
//                  NetLinx System                         //
//             Project: Bose Control Space Solution        //
//                                                         //
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
//                  Define Device                          //
/////////////////////////////////////////////////////////////
DEFINE_DEVICE

/////////////////////////////////////////////////////////////
//                  Debug	                           //
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
//                  Constants                              //
/////////////////////////////////////////////////////////////
DEFINE_CONSTANT

// Number of rings before pickup..
integer cPSTNAaNumberOfRings = 5

// Volume Step..
integer cPSTNOutStep = 2
/////////////////////////////////////////////////////////////
//                  Structures                             //
/////////////////////////////////////////////////////////////
DEFINE_TYPE

/////////////////////////////////////////////////////////////
//                  Variables                              //
/////////////////////////////////////////////////////////////
DEFINE_VARIABLE

VOLATILE INTEGER btnPSTNDialer[] =
{
    1,	// Keypad Button : 1
    2,	// Keypad Button : 2
    3,	// Keypad Button : 3
    4,	// Keypad Button : 4	
    5,	// Keypad Button : 5
    6,	// Keypad Button : 6
    7,	// Keypad Button : 7
    8,	// Keypad Button : 8
    9,	// Keypad Button : 9
    10,	// Keypad Button : 0
    11,	// Keypad Button : *
    12,	// Keypad Button : #
    13,	// Clear
    14, // Dial
    15, // Answer
    16, // End
    29, // Transfer
    30, // Backspace
    17	// Re-Dial
}


VOLATILE INTEGER btnPSTNStatus[] = 
{
    6 , // Number Display
    7 , // Caller ID 
    9 , // Status
    99, // < Caller Number - NOT USED >
    1 , // Volume
    3 , // Auto Answer
    4 , // Country Code
    10  // Account Status
}

// Misc PSTN buttons
integer btnPSTNAutoAnswer = 18
integer btnPSTNPrivacy = 19
integer btnPSTNPhoneBook = 20
integer lvlPSTNIncall = 31

// Auto Answer Enabled..
integer PSTNAutoAnswer[3]

// Address book offest tracking..
integer PSTNAbOffset = 0

// Level Control..
integer btnPSTNLvl = 1
integer btnPSTNFbLvl = 11

// Mute Control..
integer btnPSTNMute = 1
integer PSTNMuteMap = 3		// Map mute button to module channel..

// Level Control Buttons..
integer btnPSTNLvlUp = 11
integer btnPSTNLvlDn = 21

// Outgoing level tracker..
integer pstnOutLevel[3]
integer pstnOutMute[3]

/////////////////////////////////////////////////////////////
//                  Functions/Calls                        //
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
// Update Auto Answer Feedback..
define_function fnUpdatePSTNAutoAnswerUi () {
    if(PSTNAutoAnswer[RoomControling]) {
	on[dvTpPSTN,btnPSTNAutoAnswer]
    }
    else {
	off[dvTpPSTN,btnPSTNAutoAnswer]
    }
}

/////////////////////////////////////////////////////////////
// 
define_function fnDisplayPSTNAddressBook () {
    PSTNAbOffset = 0
    fnDisplayPSTNPbEntries(1)
    send_command dvTpPSTN,'PPON-PSTN-PhoneBook'
}

define_function fnDisplayPSTNPbEntries(integer start) {
    integer i;

    for (i = 0; i <= cMaxPbEntriesPerPage - 1; i++) {
	send_command dvTpPSTN,"'^BMF-',itoa(btnAbEntries[i+ 1]),',1&2,%T',PSTN_pb.entrie[i + start].name"
    }
}

/////////////////////////////////////////////////////////////
// fnUpdatePSTNUi
define_function fnUpdatePSTNUi () {
    fnUpdatePSTNAutoAnswerUi ()
    [dvTpPSTNOut,btnPSTNMute] = pstnOutMute[RoomControling]
    send_level dvTpPSTNOut,btnPSTNFbLvl,pstnOutLevel[RoomControling]
}
/////////////////////////////////////////////////////////////
//                  StartUp                                //
/////////////////////////////////////////////////////////////
DEFINE_START

/////////////////////////////////////////////////////////////
//                  Event's                                //
/////////////////////////////////////////////////////////////
DEFINE_EVENT

/////////////////////////////////////////////////////////////
// Channel Events..
channel_event[vdvSystem,1] {
    on: { 
	// Start Module..
	on[vdvPSTN,1]
	on[vdvPSTNOut,1]
    }
}
/////////////////////////////////////////////////////////////
// Data Events..

/////////////////////////////////////////////////////////////
// Button Events..
button_event[dvTpPSTN,btnPSTNAutoAnswer] {
    push: {

	if(PSTNAutoAnswer[RoomControling]) {
	    // Auto Answer Off
	    send_level vdvPSTNRm[RoomControling],2,0
	}
	else {
	    // Auto Answer On
	    send_level vdvPSTNRm[RoomControling],2,cPSTNAaNumberOfRings
	}
    }
}

button_Event[dvTpPSTN,btnPSTNPhoneBook] {
    push: {
	fnDisplayPSTNAddressBook ()
    }
}

button_event[dvTpPSTN,btnAbArrows] {
    push: {
    	integer btn;
	
	btn = get_last(btnAbArrows)
	
	switch(btn) {
	    // Up Arrow..
	    case 1: {
		if(PSTNAbOffset - cMaxPbEntriesPerPage > 0 && PSTNAbOffset - cMaxPbEntriesPerPage <= PSTN_pb.totalEntries) {
		    PSTNAbOffset = PSTNAbOffset - cMaxPbEntriesPerPage
		}
		else {
		    PSTNAbOffset = 1
		}
	    }
	    
	    // Down Arrow..
	    case 2: {
		if(PSTNAbOffset + cMaxPbEntriesPerPage <= PSTN_pb.totalEntries) {
		    PSTNAbOffset = PSTNAbOffset + cMaxPbEntriesPerPage
		}
	    }
	}
	
	fnDisplayPSTNPbEntries(PSTNAbOffset)
    }
}

button_event[dvTpPSTN,btnAbEntries] {
    push: {
	integer btn;
	
	btn = get_last(btnAbEntries)
	
	send_string vdvPSTNRm[RoomControling],"'DIAL-NUMBER=',PSTN_pb.entrie[PSTNAbOffset + btn].number"
	send_command dvTpPSTN,'PPOF-PSTN-PhoneBook'
    }
}

/////////////////////////////////////////////////////////////
// Level Events..
// Auto Answer feedback..
level_event[vdvPSTNRm,2] {
    integer room;
    
    room = get_last(vdvPSTNRm)
    PSTNAutoAnswer[room] = level.value
    
    if(room = RoomControling) {
	fnUpdatePSTNAutoAnswerUi ()
    }
}

/////////////////////////////////////////////////////////////
// Channel Events..
// Call Active..
channel_event[vdvPSTNRm,21] {
    on:{ 
	integer room;
	
	room = get_last(vdvPSTNRm)
	
	if(room = RoomControling) {
	    send_command dvTpPSTN,"'^SHO-',itoa(lvlPSTNIncall),',0'"
	}
    }
    off: {
    	integer room;
	
	room = get_last(vdvPSTNRm)
	
	if(room = RoomControling) {
	    send_command dvTpPSTN,"'^SHO-',itoa(lvlPSTNIncall),',1'"
	}
    }
}
// Ringing..
channel_event[vdvPSTNRm,22] {
    on:{ 
	integer room;
	
	room = get_last(vdvPSTNRm)
	
	if(room = RoomControling) {
	    send_command dvTpPSTN,'PPON-PSTN-Call'
	}
	
    }
    off: {
	integer room;
	
	room = get_last(vdvPSTNRm)
	
	if(room = RoomControling) {
	    send_command dvTpPSTN,'PPOF-PSTN-Call'
	}
	
    }
}

/////////////////////////////////////////////////////////////
// Data Events..
// The following events pass data between the UIDialer and the PSTN Module..
data_event[vdvPSTNRouter] {
    command: {
	send_command vdvPSTNRm[RoomControling],data.text
    }

}

data_event[vdvPSTNRm] {
    string: {
	integer room;
	
	room = get_last(vdvPSTNRm)
	
	if(room = RoomControling) {
	    send_string vdvPSTNRouter,data.text
	}
    }
}

/////////////////////////////////////////////////////////////
// PSTN Out functionality..
/////////////////////////////////////////////////////////////

// Send Level to module..
level_event[dvTpPSTNOut,btnPSTNLvl] {
    if(tpOnline) {
	send_level vdvPSTNOutRm[RoomControling],btnPSTNLvl,level.value
    }
}

// Update Panel Feedback..
level_event[vdvPSTNOutRm,btnPSTNLvl] {

    integer lvl,room;
    
    room = get_last(vdvPSTNOutRm);
    
    if(room = RoomControling) {
	send_level dvTpPSTNOut,btnPSTNFbLvl,level.value
    }
    
    pstnOutLevel[room] = level.value
}

button_event[dvTpPSTNOut,btnPSTNMute] {
    push: {
	do_push(vdvPSTNOutRm[RoomControling],PSTNMuteMap)
    }
}

channel_event[vdvPSTNOutRm,PSTNMuteMap] {
    on: {
	integer room;
    
	room = get_last(vdvPSTNOutRm);
	
	if(room = RoomControling) {
	    on[dvTpPSTNOut,btnPSTNMute]
	}
	pstnOutMute[room] = 1
    }
    off: {
	integer room;
    
	room = get_last(vdvPSTNOutRm);
	
	if(room = RoomControling) {
	    off[dvTpPSTNOut,btnPSTNMute]
	}
	pstnOutMute[room] = 0
    }
}

button_event[dvTpPSTNOut,btnPSTNLvlUp] {
    push: {
	if(PSTNOutLevel[RoomControling] + cPSTNOutStep <= 255) {
	    send_level vdvPSTNOutRm[RoomControling],btnPSTNLvl,PSTNOutLevel[RoomControling] + cPSTNOutStep
	}
    }
}


button_event[dvTpPSTNOut,btnPSTNLvlDn] {
    push: {
	if(PSTNOutLevel[RoomControling] - cPSTNOutStep <= 255) {
	    send_level vdvPSTNOutRm[RoomControling],btnPSTNLvl,PSTNOutLevel[RoomControling] - cPSTNOutStep
	}
    }
}

/////////////////////////////////////////////////////////////
//                  MainLine                               //
/////////////////////////////////////////////////////////////
DEFINE_PROGRAM

/////////////////////////////////////////////////////////////
//                  The End                                //
/////////////////////////////////////////////////////////////

